/***********************************************************************
 *			AT25SPI.C
 *			Library for memory EEPROM AT25XXX
 *			Interface SPI
 *			This program is free software; 
 *			Original code by Petr Krc ptkr@seznam.cz
 *			Tested with AT90S8515 and ATMEGA163 & AT25640
 *			Compiled using avr-gcc
 *			No warranty
 *
 *	Code implemented to EEPROMs with Byte Write Operation:
 *			- AT25640	(8K)	(32-byte Page Mode)
 *			- AT25256	(32K)	(64-byte Page Mode)
 *			- etc
 ***********************************************************************/
#include <interrupt.h>
#include "at25spi.h"
#include "ioreg.h"

/***************************************************************
 *			Constants
 ***************************************************************/
// Define EEPROM commands
#define  EE_READ              3
#define  EE_WRITE             2
#define  EE_WREN              6
#define  EE_WRDI              4	
#define  EE_RDSR              5	
#define  EE_WRSR              1

// Define EEPROM status
#define SS_WRITING			0x01	// Write cycle in progress
#define SS_WRITE_ENABLE		0x02	// Write enabled

// Define EEPROM pins
#define  pin_EE_CS       (1<<4)
#define  pin_EE_IO       (1<<5)
#define  pin_EE_CL       (1<<7)

/***************************************************************
 *			Structures
 ***************************************************************/

/***************************************************************
 *			Data
 ***************************************************************/
BYTE PAGE_SIZE;

/***************************************************************
 *			Prototypes
 ***************************************************************/

/**************************************************************************
 *	Initialize 
 **************************************************************************/
PUBLIC BOOL EE_init(BYTE page_mode)
{
	rSPCR = BV(SPE) | BV(MSTR) | BV(SPR0);	// SPI enable, master mode, CLK/16
	rDDRB |= pin_EE_CS | pin_EE_CL | pin_EE_IO;	// rPORTB input/output
	rPORTB |= pin_EE_CS;				// rPORTB no pull-ups, EE_CS high
	
	PAGE_SIZE = page_mode;

	return TRUE;
}

/**************************************************************************
 *	Wait for end of SPI transfer
 **************************************************************************/
PRIVATE char wait_for_SPI(void)	
{
	while (!(rSPSR & BV(SPIF))); // Peligro !!!??? a veces no sale
								 // despues de un comando de EE_WRITE
	return rSPDR;
}

/**************************************************************************
 *	Send command to external EEPROM
 **************************************************************************/
PRIVATE void EE_command(char cmd)
{
	rPORTB |= pin_EE_CS;				// EE_CS = 1
 	rSPCR  &= ~(BV(DORD)) & ~(BV(CPHA)) & ~(BV(CPOL));	// MSB first, phase and polarity
	rPORTB &= ~pin_EE_CS;				// EE_CS = 0
	rSPDR   = cmd;						// Command
}

/**************************************************************************
 *	Read status byte from external EEPROM
 **************************************************************************/
PRIVATE unsigned char EE_read_status(void)
{
	EE_command(EE_RDSR);				// Command EE_RDSR
	wait_for_SPI();
	rSPDR   = 0xff;						// Dummy byte
	wait_for_SPI();
	rPORTB |= pin_EE_CS;				// EE_CS = 1 (end of transmision)
	return rSPDR;
}

/**************************************************************************
 *	Read 1 byte from external EEPROM
 **************************************************************************/
PUBLIC BYTE EE_read_byte(WORD adr)		
{
	EE_command(EE_READ);				// Command EE_READ
	wait_for_SPI();
	rSPDR   = (adr >> 8);				// Adress high byte
	wait_for_SPI();
	rSPDR   = adr;						// Adress low byte
	wait_for_SPI();
	rSPDR   = adr;						// Dummy byte
	wait_for_SPI();
	rPORTB |= pin_EE_CS;				// EE_CS = 1 (end of transmision)
	return rSPDR;
}

/**************************************************************************
 *	Read a buffer from external EEPROM
 **************************************************************************/
PUBLIC void EE_read(WORD adr, BYTE *bf, WORD len)
{
	EE_command(EE_READ);				// Command EE_READ
	wait_for_SPI();
	rSPDR   = (adr >> 8);				// Adress high byte
	wait_for_SPI();
	rSPDR   = adr;						// Adress low byte
	wait_for_SPI();
	while (len--) {
		rSPDR  = adr;					// Dummy byte
		wait_for_SPI();
		*bf++ = rSPDR;					// Readed byte
	}
	rPORTB |= pin_EE_CS;				// EE_CS = 1 (end of transmision)
}

/**************************************************************************
 *	Write a byte to external EEPROM
 **************************************************************************/
void EE_write_byte(WORD adr, BYTE data)
{									// Write 1 byte to external EEPROM
	EE_command(EE_WREN);				// Command EE_WREN
	wait_for_SPI();
	rPORTB |= pin_EE_CS;				// EE_CS = 1 (necessary for WREN)

	EE_command(EE_WRITE);				// Command EE_WRITE
	wait_for_SPI();		

	rSPDR   = (adr >> 8);				// Adress high byte
	wait_for_SPI();
	rSPDR   = adr;						// Adress low byte
	wait_for_SPI();
	rSPDR   = data;						// Data byte
	wait_for_SPI();
	rPORTB |= pin_EE_CS;				// EE_CS = 1 (end of transmision)


	while(1) {
		register BYTE s;

		s = EE_read_status();
		if (s & SS_WRITING)
			continue;
		break;
	}
}

/**************************************************************************
 *	Write len bytes to external EEPROM
 **************************************************************************/
PUBLIC void EE_write(WORD adr, BYTE *bf, WORD len)
{
	register BYTE cont;
	char fin;

	fin = FALSE;
	while(!fin) {
		/*-------------------------------*
		 *	Command EE_WREN
		 *-------------------------------*/
		EE_command(EE_WREN);				
		wait_for_SPI();
		rPORTB |= pin_EE_CS;	// EE_CS = 1 (necessary for WREN)

		/*-------------------------------*
		 *	Command EE_WRITE
		 *-------------------------------*/
		EE_command(EE_WRITE);
		wait_for_SPI();
		rSPDR   = (adr >> 8);	// Adress high byte
		wait_for_SPI();
		rSPDR   = adr;			// Adress low byte
		wait_for_SPI();

		/*-------------------------------*
		 *	Calcula bytes to write
		 *-------------------------------*/
		if(len > PAGE_SIZE) {
			cont = PAGE_SIZE;
			len -= PAGE_SIZE;
			adr += PAGE_SIZE;
		} else {
			cont = len;
			fin = TRUE;
		}

		/*-------------------------------*
		 *	Write data
		 *-------------------------------*/
		while(cont--) {	
			rSPDR   = *bf;				// Data byte
			bf++;
			wait_for_SPI();
		}
		rPORTB |= pin_EE_CS;			// EE_CS = 1 (end of transmision)

		/*-------------------------------*
		 *	Wait end of write
		 *-------------------------------*/
		while(1) {	// ??? Infinite loop, I don't like. Must be have a timer
			register BYTE s;
	
			s = EE_read_status();
			if (s & SS_WRITING)
				continue;
			break;
		}
	}

	/*-------------------------------*
	 *	Command EE_WRDI
	 *-------------------------------*/
	EE_command(EE_WRDI);	
	wait_for_SPI();
	rPORTB |= pin_EE_CS;				// EE_CS = 1 
}

